"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Watcher = void 0;
const utils_1 = require("@code-engine/utils");
const chokidar = require("chokidar");
const path_1 = require("path");
const create_file_1 = require("./create-file");
/**
 * Internal class that wraps the filesystem watch functionality.
 */
class Watcher {
    constructor(engine, path, config) {
        this.disposed = false;
        this.chokidar = createChokidar(path, config);
        this.output = new utils_1.IterableWriter();
        this.config = config;
        this.dir = path.dir;
        this.engine = engine;
        this.chokidar.on("add", this.changeDetected.bind(this, "created" /* Created */)); // eslint-disable-line @typescript-eslint/no-misused-promises
        this.chokidar.on("change", this.changeDetected.bind(this, "modified" /* Modified */)); // eslint-disable-line @typescript-eslint/no-misused-promises
        this.chokidar.on("unlink", this.changeDetected.bind(this, "deleted" /* Deleted */)); // eslint-disable-line @typescript-eslint/no-misused-promises
        this.chokidar.on("error", this.errorHandler.bind(this)); // eslint-disable-line @typescript-eslint/no-misused-promises
    }
    get iterable() {
        return this.output.iterable;
    }
    /**
     * Disposes the watcher. It cannot be used after this.
     */
    async dispose() {
        if (!this.disposed) {
            this.disposed = true;
            // Don't await on this promise, since we don't know or care whether
            // the rest of the output stream will be read.
            this.output.end(); // eslint-disable-line @typescript-eslint/no-floating-promises
            await this.chokidar.close();
        }
    }
    /**
     * A filesystem change was detected within the directory path.
     */
    async changeDetected(change, path, stats) {
        try {
            this.engine.log.debug(`Change detected: ${change} ${path}`, { change, dir: this.dir, path });
            let absolutePath = path_1.join(this.dir, path);
            let file = create_file_1.createChangedFile(path, absolutePath, stats, change);
            if (this.config.filter(file, this.engine)) {
                if (change !== "deleted" /* Deleted */) {
                    file.contents = await this.config.fs.promises.readFile(path_1.join(this.dir, path));
                }
                await this.output.write(file);
            }
        }
        catch (error) {
            await this.errorHandler(error);
        }
    }
    async errorHandler(error) {
        if (!this.disposed) {
            // Re-throw the error into the iterable
            await this.output.throw(error);
        }
    }
}
exports.Watcher = Watcher;
/**
 * Creates a Chokidar filesystem watcher.
 */
function createChokidar(path, config) {
    let whatToWatch, disableGlobbing;
    if (typeof config.filterCriteria === "string") {
        // The filter criteria is a glob pattern, so pass it along to Chokidar.
        whatToWatch = globify(path.dir, config.filterCriteria);
        disableGlobbing = false;
    }
    else {
        whatToWatch = path.dir;
        disableGlobbing = true;
    }
    return chokidar.watch(whatToWatch, {
        ignoreInitial: true,
        cwd: path.dir,
        disableGlobbing,
        alwaysStat: true,
        depth: config.depth,
    });
}
/**
 * Combines the specified directory path and glob pattern.
 */
function globify(dir, glob) {
    if (process.platform === "win32") {
        dir = dir.replace(/\\/g, "/");
    }
    return path_1.posix.join(dir, glob);
}
//# sourceMappingURL=watcher.js.map